use glutin::{
    event::{Event, WindowEvent as GlutinWindowEvent},
    event_loop::{ControlFlow, EventLoop},
    platform::desktop::EventLoopExtDesktop,
};
use shrev::EventChannel;
use specs::World;

pub struct Events {
    event_loop: EventLoop<()>,
}

impl Events {
    pub fn new(world: &mut World) -> Self {
        world.insert(EventChannel::<WindowEvent>::default());

        Self {
            event_loop: EventLoop::new(),
        }
    }
}

impl Events {
    pub fn handle(&self) -> &EventLoop<()> {
        &self.event_loop
    }

    pub fn process(&mut self, world: &World) {
        let mut channel = world.fetch_mut::<EventChannel<WindowEvent>>();

        self.event_loop.run_return(|event, _target, flow_control| {
            *flow_control = ControlFlow::Poll;

            match event {
                Event::MainEventsCleared | Event::RedrawRequested(_) => {
                    *flow_control = ControlFlow::Exit;
                }
                Event::WindowEvent { event, .. } => match event {
                    GlutinWindowEvent::Resized(pos) => {
                        channel.single_write(WindowEvent::Resize(pos.width, pos.height));
                    }
                    GlutinWindowEvent::CloseRequested => {
                        channel.single_write(WindowEvent::Close);
                    }
                    _ => (),
                },
                _ => (),
            }
        });
    }
}

pub enum WindowEvent {
    Resize(u32, u32),
    Close,
}
