use glc::{
    misc::Bindable,
    shader::{Program, Type},
};
use log::error;
use specs::{ReadExpect, System, World, WriteExpect};

use crate::Error;

use super::super::{
    misc::WorldHelper,
    resources::{Camera, Geometry, State},
};

pub struct Init {
    program: Program,
    resolution: (u32, u32),
}

impl Init {
    pub fn new(world: &mut World) -> Result<Self, Error> {
        let program = world.load_program(vec![
            (Type::Vertex, "resources/shader/noise.vert"),
            (Type::Fragment, "resources/shader/noise.frag"),
        ])?;

        Ok(Self {
            program,
            resolution: (0, 0),
        })
    }
}

impl<'a> System<'a> for Init {
    type SystemData = (
        WriteExpect<'a, Camera>,
        ReadExpect<'a, State>,
        ReadExpect<'a, Geometry>,
    );

    fn run(&mut self, (mut camera, state, geometry): Self::SystemData) {
        if self.resolution != state.resolution {
            self.resolution = state.resolution;

            gl::viewport(0, 0, self.resolution.0 as _, self.resolution.1 as _);

            if let Err(err) = camera.resize(self.resolution.0 as _, self.resolution.1 as _) {
                error!("Error while updating camera: {}", err);
                panic!("Error while updating camera: {}", err);
            }
        }

        self.program.bind();
        geometry.render_quad();
        self.program.unbind();
    }
}
