use glc::{
    matrix::{Angle, Matrix4f},
    misc::Bindable,
    shader::{Program, Type, Uniform},
    texture::{FilterMag, FilterMin, Texture},
    vector::Vector3f,
};
use specs::{ReadExpect, System, World};

use crate::{server::resources::Global, Error};

use super::super::{
    misc::WorldHelper,
    resources::{Camera, Geometry},
};

pub struct Test {
    model: Matrix4f,
    program: Program,
    texture: Texture,
}

impl Test {
    pub fn new(world: &World) -> Result<Self, Error> {
        let model = Matrix4f::scale(Vector3f::new(500.0, 500.0, 500.0));

        let program = world.load_program(vec![
            (Type::Vertex, "resources/shader/quad.vert"),
            (Type::Fragment, "resources/shader/quad.frag"),
        ])?;
        program.bind();
        world.resource::<Camera>()?.bind(0)?;
        program.unbind();

        let mut texture = world.load_texture("resources/textures/test.png")?;
        texture.set_filter(FilterMin::LinearMipmapLinear, FilterMag::Linear)?;

        Ok(Self {
            model,
            program,
            texture,
        })
    }
}

impl<'a> System<'a> for Test {
    type SystemData = (ReadExpect<'a, Global>, ReadExpect<'a, Geometry>);

    fn run(&mut self, (global, geometry): Self::SystemData) {
        self.model = self.model
            * Matrix4f::rotate(
                Vector3f::new(0.0, 0.0, 1.0),
                Angle::Deg(10.0 * global.delta),
            );

        self.texture.bind();
        self.program.bind();
        self.program.uniform(1, Uniform::Matrix4f(&self.model));

        geometry.render_quad();

        self.program.unbind();
        self.texture.unbind();
    }
}
