#![allow(dead_code)]

use glc::{
    array_buffer::{ArrayBuffer, Target, Usage},
    error::Error,
    matrix::Matrix4f,
};

pub struct Camera {
    buffer: ArrayBuffer,
}

impl Camera {
    pub fn new() -> Result<Self, Error> {
        let mut buffer = ArrayBuffer::new(Target::UniformBuffer)?;
        buffer.buffer_data(
            Usage::StaticDraw,
            &[Data {
                proj_world: Matrix4f::identity(),
                proj_ui: Matrix4f::identity(),
                view: Matrix4f::identity(),
            }],
        )?;

        Ok(Self { buffer })
    }

    pub fn resize(&mut self, w: f32, h: f32) -> Result<(), Error> {
        let mut data = self.buffer.map_mut::<Data>(true)?;

        data[0].proj_world = Matrix4f::ortho(-w / 2.0, w / 2.0, -h / 2.0, h / 2.0, -100.0, 100.0);
        data[0].proj_ui = Matrix4f::ortho(0.0, w, h, 0.0, -100.0, 100.0);

        Ok(())
    }

    pub fn bind(&self, index: gl::GLuint) -> Result<(), Error> {
        Error::checked(|| self.buffer.bind_buffer_base(index))
    }
}

#[repr(C, packed)]
struct Data {
    proj_world: Matrix4f,
    proj_ui: Matrix4f,
    view: Matrix4f,
}
