use std::mem::size_of;

use glc::{
    array_buffer::{ArrayBuffer, Target, Usage},
    misc::Bindable,
    vertex_array::{DataType, VertexArray},
};

use crate::Error;

pub struct Geometry {
    quad: VertexArray,
}

impl Geometry {
    pub fn new() -> Result<Self, Error> {
        Ok(Self {
            quad: create_quad_array()?,
        })
    }

    pub fn render_quad(&self) {
        self.quad.bind();

        gl::draw_arrays(gl::TRIANGLE_FAN, 0, 4);

        self.quad.unbind();
    }
}

#[repr(C, packed)]
#[derive(Debug, Copy, Clone, PartialEq)]
struct Vertex {
    pub x: f32,
    pub y: f32,
}

fn create_quad_array() -> Result<VertexArray, Error> {
    let vertices = &[
        Vertex { x: -0.5, y: -0.5 },
        Vertex { x: 0.5, y: -0.5 },
        Vertex { x: 0.5, y: 0.5 },
        Vertex { x: -0.5, y: 0.5 },
    ];

    const STRIDE_POS: gl::GLsizei = size_of::<Vertex>() as gl::GLsizei;
    const OFFSET_POS: gl::GLsizei = 0;

    let mut array_buffer = ArrayBuffer::new(Target::ArrayBuffer)?;
    array_buffer.buffer_data(Usage::StaticDraw, vertices)?;

    let vertex_array = VertexArray::builder()
        .bind_buffer(array_buffer)
        .vertex_attrib_pointer(0, 2, DataType::Float, false, STRIDE_POS, OFFSET_POS)?
        .build()?;

    Ok(vertex_array)
}
