use shrev::{EventChannel, ReaderId};
use specs::{ReadExpect, System, World, Write};

use crate::Error;

use super::super::{
    misc::{WindowEvent, WorldHelper},
    resources::State,
};

pub struct StateUpdate {
    window_events_id: ReaderId<WindowEvent>,
}

impl StateUpdate {
    pub fn new(world: &mut World) -> Result<Self, Error> {
        let window_events_id = world.register_event_reader::<WindowEvent>()?;

        Ok(Self { window_events_id })
    }
}

impl<'a> System<'a> for StateUpdate {
    type SystemData = (Write<'a, State>, ReadExpect<'a, EventChannel<WindowEvent>>);

    fn run(&mut self, (mut state, window_events): Self::SystemData) {
        let events = window_events.read(&mut self.window_events_id);
        for event in events {
            match event {
                WindowEvent::Resize(w, h) => {
                    state.resolution = (*w, *h);
                }
                WindowEvent::Close => {
                    state.close_requested = true;
                }
            }
        }
    }
}
