use log::{error, info};
use specs::{World, WorldExt};

use space_crush::{init_logger, misc::Vfs, App, Error, Server};

fn main() -> Result<(), Error> {
    let vfs = Vfs::new()?;

    init_logger(&vfs);

    info!("Application started");

    if let Err(err) = run(vfs) {
        error!("Error while executing application: {}", err);

        return Err(err);
    }

    info!("Application exited");

    Ok(())
}

fn run(vfs: Vfs) -> Result<(), Error> {
    let mut world = World::new();
    world.insert(vfs);

    let mut server = Server::new(&mut world);
    let mut app = App::new(&mut world)?;

    while app.is_running() {
        server.process(&world);
        app.process(&world)?;
    }

    Ok(())
}
