use std::io::Error as IoError;

use glc::error::Error as GlcError;
use glutin::{ContextError as GlutinContextError, CreationError as GlutinCreationError};
use glyph_brush::ab_glyph::InvalidFont;
use serde_yaml::Error as YmlError;
use thiserror::Error;
use vfs::VfsError;
use vfs_zip::Error as VfsZipError;

#[derive(Debug, Error)]
pub enum Error {
    #[error("IO Error: {0}")]
    IoError(IoError),

    #[error("VFS Error: {0}")]
    VfsError(VfsError),

    #[error("VFS ZIP Error: {0}")]
    VfsZipError(VfsZipError),

    #[error("YML Error: {0}")]
    YmlError(YmlError),

    #[error("GLC Error: {0}")]
    GlcError(GlcError),

    #[error("glutin Context Error: {0}")]
    GlutinContextError(GlutinContextError),

    #[error("glutin Creation Error: {0}")]
    GlutinCreationError(GlutinCreationError),

    #[error("Invalid Font: {0}")]
    InvalidFont(InvalidFont),

    #[error("Resource is not registered: {0}!")]
    ResourceNotRegistered(&'static str),

    #[error("Unable to create OpenGL context!")]
    CreateContext,

    #[error("Unable to initialize VFS!")]
    InitVFS,

    #[error("Font is not set!")]
    FontNotSet,
}

impl From<IoError> for Error {
    fn from(err: IoError) -> Self {
        Self::IoError(err)
    }
}

impl From<VfsError> for Error {
    fn from(err: VfsError) -> Self {
        Self::VfsError(err)
    }
}

impl From<VfsZipError> for Error {
    fn from(err: VfsZipError) -> Self {
        Self::VfsZipError(err)
    }
}

impl From<YmlError> for Error {
    fn from(err: YmlError) -> Self {
        Self::YmlError(err)
    }
}

impl From<GlcError> for Error {
    fn from(err: GlcError) -> Self {
        Self::GlcError(err)
    }
}

impl From<GlutinContextError> for Error {
    fn from(err: GlutinContextError) -> Self {
        Self::GlutinContextError(err)
    }
}

impl From<GlutinCreationError> for Error {
    fn from(err: GlutinCreationError) -> Self {
        Self::GlutinCreationError(err)
    }
}

impl From<InvalidFont> for Error {
    fn from(err: InvalidFont) -> Self {
        Self::InvalidFont(err)
    }
}
